/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef CONFIGARCHIVEACCESSSVC_H
#define CONFIGARCHIVEACCESSSVC_H

// Include files
#include <memory>
#include <string>
// from Gaudi
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/Service.h"

#include "Kernel/IConfigAccessSvc.h"

/** @class ConfigFileAccessSvc ConfigFileAccessSvc.h
 *
 *  functionality:
 *        read/write configure information to files
 *
 *  @author Gerhard Raven
 *  @date   2009-06-04
 */

class IArchive;

class ConfigArchiveAccessSvc : public extends<Service, IConfigAccessSvc> {
public:
  ConfigArchiveAccessSvc( const std::string& name, ISvcLocator* pSvcLocator );

  StatusCode initialize() override; ///< Service initialization

  std::optional<PropertyConfig> readPropertyConfig( const PropertyConfig::digest_type& ref ) override;
  PropertyConfig::digest_type   writePropertyConfig( const PropertyConfig& config ) override;

  std::optional<ConfigTreeNode> readConfigTreeNode( const ConfigTreeNode::digest_type& ref ) override;
  ConfigTreeNode::digest_type   writeConfigTreeNode( const ConfigTreeNode& config ) override;

  std::optional<ConfigTreeNode>   readConfigTreeNodeAlias( const ConfigTreeNodeAlias::alias_type& ) override;
  ConfigTreeNodeAlias::alias_type writeConfigTreeNodeAlias( const ConfigTreeNodeAlias& ) override;

  std::vector<ConfigTreeNodeAlias> configTreeNodeAliases( const ConfigTreeNodeAlias::alias_type& ) override;

private:
  virtual IArchive* file() const = 0;

  template <typename T>
  std::optional<T> read( const std::string& path ) const;
  template <typename T>
  bool        write( const std::string& path, const T& object ) const;
  std::string propertyConfigPath( const PropertyConfig::digest_type& digest ) const;
  std::string configTreeNodePath( const ConfigTreeNode::digest_type& digest ) const;
  std::string configTreeNodeAliasPath( const ConfigTreeNodeAlias::alias_type& alias ) const;
};
#endif // CONFIGTARFILEACCESSSVC_H
