###############################################################################
# (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""
Implementation of the validation code for all the DQFilter tests.
"""
from builtins import zip
from builtins import map
__author__ = "Marco Clemencic <marco.clemencic@cern.ch>"

import re
from pprint import pformat

from GaudiTesting.BaseTest import cmpTreesDicts, getCmpFailingValues


def count(keys, lines):
    """Helper function to count how many times a substring is found in a list of strings.
    """
    result = dict(zip(keys, [0] * len(keys)))
    for l in lines:
        for k in keys:
            if k in l:
                result[k] += 1
                continue
    return result


def checkEff(name, expected, stdout, result, causes):
    eff = None
    name = name.strip('"')
    qname = '"%s"' % (name)
    for l in stdout.splitlines():
        if qname in l:
            eff = [_f.strip() for _f in l.split("|") if _f.strip()]
            eff = tuple(map(int, eff[1:3]))
            break

    result["GaudiTest.%s.expected" % name] = result.Quote(pformat(expected))

    if eff != expected:
        causes.append(name)
        result["GaudiTest.%s.found" % name] = result.Quote(pformat(eff))


def checkCounts(expected, stdout, result, causes):
    ## Check DQFilter messages
    dqf = re.compile(r"^(DQFilter|ToolSvc\.DQAccep)")
    lines = [l.rstrip() for l in stdout.splitlines() if dqf.match(l)]

    counts = count(list(expected.keys()), lines)

    result["GaudiTest.DQFilterOut.expected"] = result.Quote(pformat(expected))

    failed = cmpTreesDicts(expected, counts)
    if failed:
        causes.append("DQFilter output")
        result["GaudiTest.DQFilterOut.found"] = result.Quote(pformat(counts))
        msg = "%s: %s != %s" % getCmpFailingValues(expected, counts, failed)
        result["GaudiTest.DQFilterOut.failure_on"] = result.Quote(msg)
