/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef LOKI_COREENGINE_H
#define LOKI_COREENGINE_H 1
// ============================================================================
// Include files
// ============================================================================
// LoKi
// ============================================================================
#include "LoKi/Context.h"
#include "LoKi/ICoreAntiFactory.h"
// ============================================================================
/** @file LoKi/CoreEngine.h
 *
 *  This file is a part of LoKi project -
 *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
 *
 *  The package has been designed with the kind help from
 *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas,
 *  contributions and advices from G.Raven, J.van Tilburg,
 *  A.Golutvin, P.Koppenburg have been used in the design.
 *
 *  @author Vanya BELYAEV Ivan.Belyaev@itep.ru
 */
namespace LoKi {
  // ==========================================================================
  namespace Hybrid {
    // ========================================================================
    /** @class CoreEngine LoKi/CoreEngine.h
     *
     *  Helper class for implementation of C++/Python "Hybrid" solution
     *
     *  This file is a part of LoKi project -
     *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
     *
     *  The package has been designed with the kind help from
     *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas,
     *  contributions and advices from G.Raven, J.van Tilburg,
     *  A.Golutvin, P.Koppenburg have been used in the design
     *
     *  @author Vanya BELYAEV Ivan.Belyaev@nikhef.nl
     *  @date   2008-09-18
     */
    class CoreEngine {
    public:
      // ======================================================================
      /// Standard constructor
      CoreEngine() = default;
      // ======================================================================
      // destructor
      virtual ~CoreEngine() = default; // Destructor
      // ======================================================================
    public: // "void -> ... "
      // ======================================================================
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::FCuts& cut ) const;
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::FFunc& cut ) const;
      // ======================================================================
    public: // "double -> ... "
      // ======================================================================
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::XCuts& cut ) const;
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::XFunc& cut ) const;
      // ======================================================================
    public: // some functional stuff
      // ======================================================================
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::XMaps& cut ) const;
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::XFunVals& cut ) const;
      // ======================================================================
      /// propagate the cut to the tool
      StatusCode process( const std::string& name, const LoKi::Types::XSources& cut ) const;
      // ======================================================================
    public:
      // ======================================================================
      /// get the current context
      const LoKi::Context* context() const;
      // ======================================================================
    };
    // ========================================================================
  } // namespace Hybrid
  // ==========================================================================
} // end of namespace LoKi
// ============================================================================
// The END
// ============================================================================
#endif // LOKI_COREENGINE_H
