/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once
#include "Event/UTCluster.h"
#include "Event/UTDigit.h"
#include "GaudiKernel/SmartRefVector.h"
#include "Kernel/UTPPRepresentation.h"
#include "Kernel/UTStripRepresentation.h"
#include "UTBitsPolicy.h"
#include <utility>

namespace UTFun {

  /** quantize the interstrip fraction  from 'float' */
  double stripFraction( const double interStripPos );

  /**
   * calculate mean strip
   * @param  digits vector of digits
   * @return mean strip
   */
  std::pair<double, unsigned int> position( const SmartRefVector<LHCb::UTDigit>& digits, double trim = 0 );

  /**
   * calculate mean strip
   * @param  strips vector of strip values
   * @return mean strip
   */
  std::pair<double, unsigned int> position( const LHCb::UTCluster::ADCVector& strips, double trim = 0 );

  inline double stripFraction( const double interStripPos ) {
    return ( LHCb::Math::round( ( 1 << UTBitsPolicy::interStripPrecision ) * interStripPos ) ) /
           double( 1 << UTBitsPolicy::interStripPrecision );
  }

} // namespace UTFun
